function [XMin,FMin,FCount] = DTSps(f,U,L)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%              Directed Tabu Search (APS)                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% A. Hedar, April, 2003
%
% Reference:
%       A. Hedar and M. Fukushima, "TS Directed by direct search methods
%       for nonlinear global optimization", Technical report 2003-007, 
%       Department of Applied Mathematics and Physics,
%       Kyoto University (June 2003).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%          function [XMin,FMin,FCount] = DTSps(f,U,L)                %
% Inputs:   
%   f = Objective function 
%   U = n-Dimension Vector of upper limits of the objective variables
%   L = n-Dimension Vector of lower limits of the objective variables
%
% Outputs:
%   XMin = Best point obtained by DTSps
%   FMin = Best function value obtained by DTSps
%	FCount = Number of function evaluations             
%
%%%%%%%%%%%%%%%%%% Intial Parameters %%%%%%%%%%%%%%%%%%%%
%
n=length(U);
Edg=0.1*min(U-L);               % Simplex edge length
FCount=0;                       % Number of function evaluations
tol =1.0D-6;                    % Termanation acurcy
MaxItrs=10*n; 
itrc=0;
NoImprv=2*n;
inNoImprv=0;
H=1;                            % Ratio of accepting Diversification point
dVR=2*Edg;                      % Region redius in VRL
%
%%%%%%%%%% Generation of the initial Solution %%%%%%%%%
%
r=rand(1,n);
x=L+(U-L).*r;
fx=feval(f,x);
FCount=FCount+1;
fbetter=fx;
S=zeros(n+2,n);
fv=zeros(n+2,1);
id=eye(n);
%
% **************** 1st iteration ******************
% Constructing the Neighborhood Search
in=1;
while(fbetter >= fx & in <= n)      
    EdgL=Edg+rand*Edg;
    sn=(-1)^(round(rand));
    S(in,:)=x+sn*EdgL*id(in,:);   
    fv(in)=feval(f,S(in,:));
    FCount=FCount+1;
    if fv(in)<fbetter
        x1=S(in,:);
        f1=fv(in);
        Bflag=1;
        fbetter=fv(in);  
    else
        Bflag=0;
    end
    in=in+1;
end
if (Bflag == 0)
    df=zeros(n,1);
    u=zeros(n,n);
    v=zeros(1,n);    
    for j=1:n;        
        df(j)=fv(j)-fx;
	end	
	for i=1:n; 
        w(i)=df(i)/sum(abs(df));
        u(i,:)=-(S(i,:)-x); 
        u(i,:)=u(i,:)/norm(u(i,:));
	end      
	for j=1:n; v=v+w(j)*u(j,:); end    
    v=v/norm(v);
    S(n+1,:)=x+(0.5*rand)*Edg*v;
    S(n+2,:)=x+(0.5+0.5*rand)*Edg*v;    
    for j=1:2
        jn=n+j;
        fv(jn)=feval(f,S(jn,:));
    end        
    FCount=FCount+2;
    [fvs,is]=sort(fv); 
    Stmp=S(is,:); S=Stmp; fv=fvs;    
    xnew=S(1,:);
    fnew=fv(1);
else
    xnew=x1; fnew=f1;    
end
% TL and VRL Setting
inTL=1; TL(inTL,:)=x; fTL(inTL)=fx; rec(inTL)=1;
inVRL=1; VRL(inVRL,:)=x; frVRL(inVRL)=1;
x=xnew; fx=fnew;
Xold=x; Fold=fx;
%%%%%%%%%%%%%%%%%%%%%% Main Loop %%%%%%%%%%%%%%%%%%%%%%%%%
%
while(inNoImprv < NoImprv & itrc < MaxItrs)    
    [FM1,XM1,FCount1,TL1,fTL1,rec1,VRL1,frVRL1]=APS(f,n,x,fx,Edg,TL,fTL,rec,VRL,frVRL);
    TL=TL1;
    fTL=fTL1;
    rec=rec1;
    VRL=VRL1;
    frVRL=frVRL1;
	FCount=FCount+FCount1;
	% Diversification Solution 
	accpt=0;
	while accpt==0        
		r=rand(1,n);
		xnw=L+(U-L).*r;
		for j=1:inVRL
            dxV(j)=norm(xnw-VRL(j,:))*(1+0.25*(1-exp(-0.25*(frVRL(j)-1))));
		end        
        if min(dxV)>= H*dVR        
            accpt=1;
        end    
	end    
	x=xnw;
	fx=feval(f,x);
	FCount=FCount+1;
	% Update TL
    %
    [sortrec,iTLr]=sort(rec);
	TLrtmp=TL(iTLr,:); TL=TLrtmp;
	fTLtmp=fTL(iTLr); fTL=fTLtmp;
	rec=sortrec;
    inTL=length(fTL);    
    for j=1:inTL
        rec(j)=j;
    end
    %
	[sortfTL,iTL]=sort(fTL);
	TLtmp=TL(iTL,:); TL=TLtmp;
	rectmp=rec(iTL); rec=rectmp;
	fTL=sortfTL;
	fTL(inTL)=fx;
	TL(inTL,:)=x;
	rec(inTL)=inTL+1;
    %
    inVRL=length(frVRL);
    VRL(inVRL+1,:)=x; frVRL(inVRL+1)=1;
    %
    if FM1 <= Fold
        inNoImprv = 0;
    else
        inNoImprv = inNoImprv+1;
    end
    Xold=XM1; Fold=FM1;      
    itrc=itrc+1;    
end
%
%%%%%%%%%%%%% Intensification Search %%%%%%%%%%%%%%%%
%
[fTLs,ifTL]=sort(fTL); 
TLtmp=TL(ifTL,:);
TL=TLtmp; fTL=fTLs;
XMin=TL(1,:); XM=TL;
FMin=fTL(1);
% Applying N-M method starting from Best Point
if n<5
    maxitt=100*n;
    budget=200*n;
else
    maxitt=5000*n;
    budget=10000*n;
end
for j=1:1
    [x1,FCoun1,f1]=NeMe(n,f,XM(j,:),maxitt,budget);
    if f1 < FMin
        XMin1=x1(:,1);
        FMin=f1;   
    end
    FCount=FCount+FCoun1;
end
%%%%%%%%%%%%%%%%%%%%%%%%%% END %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%